# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dataset_serialize.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='dataset_serialize.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x17\x64\x61taset_serialize.proto\"\x81\x01\n\x08\x44setFile\x12\r\n\x05parms\x18\x01 \x01(\t\x12\x10\n\x08\x65xamples\x18\x02 \x03(\t\x12%\n\x06images\x18\x03 \x03(\x0b\x32\x15.DsetFile.ImagesEntry\x1a-\n\x0bImagesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\x62\x06proto3'
)




_DSETFILE_IMAGESENTRY = _descriptor.Descriptor(
  name='ImagesEntry',
  full_name='DsetFile.ImagesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='DsetFile.ImagesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='DsetFile.ImagesEntry.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=112,
  serialized_end=157,
)

_DSETFILE = _descriptor.Descriptor(
  name='DsetFile',
  full_name='DsetFile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='parms', full_name='DsetFile.parms', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='examples', full_name='DsetFile.examples', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='images', full_name='DsetFile.images', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DSETFILE_IMAGESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28,
  serialized_end=157,
)

_DSETFILE_IMAGESENTRY.containing_type = _DSETFILE
_DSETFILE.fields_by_name['images'].message_type = _DSETFILE_IMAGESENTRY
DESCRIPTOR.message_types_by_name['DsetFile'] = _DSETFILE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DsetFile = _reflection.GeneratedProtocolMessageType('DsetFile', (_message.Message,), {

  'ImagesEntry' : _reflection.GeneratedProtocolMessageType('ImagesEntry', (_message.Message,), {
    'DESCRIPTOR' : _DSETFILE_IMAGESENTRY,
    '__module__' : 'dataset_serialize_pb2'
    # @@protoc_insertion_point(class_scope:DsetFile.ImagesEntry)
    })
  ,
  'DESCRIPTOR' : _DSETFILE,
  '__module__' : 'dataset_serialize_pb2'
  # @@protoc_insertion_point(class_scope:DsetFile)
  })
_sym_db.RegisterMessage(DsetFile)
_sym_db.RegisterMessage(DsetFile.ImagesEntry)


_DSETFILE_IMAGESENTRY._options = None
# @@protoc_insertion_point(module_scope)
